(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 12.3' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     74178,       1764]
NotebookOptionsPosition[     71601,       1717]
NotebookOutlinePosition[     72007,       1733]
CellTagsIndexPosition[     71964,       1730]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["CoVaR Demo\t", "Title",
 CellChangeTimes->{{3.860667638221408*^9, 
  3.8606676409836626`*^9}},ExpressionUUID->"1a68cbf2-ea48-4487-aff9-\
fc9ec31337b5"],

Cell["\<\
Covariance Root Finding Demo \[LineSeparator]Finding eigenstates of a spin \
model\
\>", "Subtitle",
 CellChangeTimes->{{3.8606676445046988`*^9, 3.860667650262312*^9}, {
  3.8606677329858084`*^9, 3.8606677606216106`*^9}, {3.9029209931399393`*^9, 
  3.9029209942408104`*^9}},ExpressionUUID->"00f3e1a7-c8ac-4c69-bccc-\
6d9ce8570bfd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"Import", " ", "QuESTlink"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
    "Import", "[", "\"\<https://qtechtheory.org/QuESTlink.m\>\"", "]"}], 
    ";"}], "\n", 
   RowBox[{
    RowBox[{"CreateDownloadedQuESTEnv", "[", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"numQs", "=", "5"}], ";"}], "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.860667669842697*^9, 3.8606677099352155`*^9}, {
  3.86066847886563*^9, 3.860668481619475*^9}, {3.860670140705469*^9, 
  3.8606701407674685`*^9}, {3.8606701944110627`*^9, 3.8606701944770627`*^9}},
 CellLabel->
  "In[289]:=",ExpressionUUID->"957f1326-ac59-4622-a6ef-d5d9fdff11e5"],

Cell[TextData[{
 "First we need to set up the problem we wish to solve, we will use a \
Heisenberg Hamiltonian that models spin interactions\n",
 Cell[BoxData[
  FormBox[
   TemplateBox[<|"boxes" -> FormBox[
       RowBox[{
         StyleBox["H", "TI"], "\[LongEqual]", 
         UnderoverscriptBox["\[Sum]", 
          RowBox[{
            StyleBox["i", "TI"], "\[LongEqual]", "1"}], 
          StyleBox["N", "TI"], LimitsPositioning -> True], 
         StyleBox["J", "TI"], 
         RowBox[{"(", 
           RowBox[{
             SubscriptBox[
              StyleBox["X", "TI"], 
              StyleBox["i", "TI"]], 
             SubscriptBox[
              StyleBox["X", "TI"], 
              RowBox[{
                StyleBox["i", "TI"], "+", "1"}]], "+", 
             SubscriptBox[
              StyleBox["Y", "TI"], 
              StyleBox["i", "TI"]], 
             SubscriptBox[
              StyleBox["Y", "TI"], 
              RowBox[{
                StyleBox["i", "TI"], "+", "1"}]], "+", 
             SubscriptBox[
              StyleBox["Z", "TI"], 
              StyleBox["i", "TI"]], 
             SubscriptBox[
              StyleBox["Z", "TI"], 
              RowBox[{
                StyleBox["i", "TI"], "+", "1"}]]}], ")"}], "+", 
         UnderoverscriptBox["\[Sum]", 
          RowBox[{
            StyleBox["i", "TI"], "\[LongEqual]", "1"}], 
          StyleBox["N", "TI"], LimitsPositioning -> True], 
         SubscriptBox[
          StyleBox["c", "TI"], 
          StyleBox["i", "TI"]], 
         SubscriptBox[
          StyleBox["Z", "TI"], 
          StyleBox["i", "TI"]]}], TraditionalForm], "errors" -> {}, "input" -> 
     "H=\\sum_{i=1}^N J (X_i X_{i+1} + Y_i Y_{i+1} + Z_i Z_{i+1}) + \
\\sum_{i=1}^N c_i Z_i", "state" -> "Boxes"|>,
    "TeXAssistantTemplate"], TraditionalForm]],ExpressionUUID->
  "9a13dd79-fd76-4188-b6ac-a2160dcb4aaf"],
 "\nand choose a parametrised circuit with a hardware-efficient form"
}], "Text",
 CellChangeTimes->{{3.860668268031686*^9, 3.8606683465532427`*^9}, {
  3.8606685100349245`*^9, 3.860668533254661*^9}, {3.8606685836984253`*^9, 
  3.8606686029225597`*^9}, {3.9029210090267344`*^9, 
  3.9029210403773527`*^9}},ExpressionUUID->"8eeba43a-a171-43fd-b1ca-\
a19cef20426b"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"freqs", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.7098301012032957`"}], ",", 
     RowBox[{"-", "0.05177915010485945`"}], ",", "0.9065658350752286`", ",", 
     RowBox[{"-", "0.9265872629174545`"}], ",", "0.09500423337423802`"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"spinHamil", "[", 
    RowBox[{"J_", ",", "numQs_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "hamil", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"hamil", "=", 
       RowBox[{"ExpandAll", "@", 
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{"J", "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SubscriptBox["X", "k"], 
              SubscriptBox["X", "l"]}], "+", 
             RowBox[{
              SubscriptBox["Y", "k"], 
              SubscriptBox["Y", "l"]}], "+", 
             RowBox[{
              SubscriptBox["Z", "k"], 
              SubscriptBox["Z", "l"]}]}], ")"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", "0", ",", 
            RowBox[{"numQs", "-", "1"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"l", ",", 
            RowBox[{"k", "+", "1"}], ",", 
            RowBox[{"numQs", "-", "1"}]}], "}"}]}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"hamil", "+=", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"freqs", "[", 
           RowBox[{"[", "k", "]"}], "]"}], "*", 
          SubscriptBox["Z", 
           RowBox[{"k", "-", "1"}]]}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "1", ",", "numQs"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "@", "hamil"}], ";"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"hamil", "=", 
   RowBox[{"spinHamil", "[", 
    RowBox[{"0.1", ",", "numQs"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"hamilmat", "=", 
   RowBox[{"CalcPauliStringMatrix", "[", "hamil", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"eigs", "=", 
    RowBox[{"Eigenvalues", "[", "hamilmat", "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"my\[Theta]", "[", "num_", "]"}], ":=", 
   SubscriptBox["\[Theta]", "num"]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"generateAnsatzCircuit", "[", 
     RowBox[{"circuitN_", ",", "nblock_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"ansatz", ",", "c", ",", "blocks", ",", "paramCount"}], "}"}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"ansatz", "=", 
        RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"c", "=", "1"}], ";", " ", "\[IndentingNewLine]", 
       RowBox[{"ansatz", "=", 
        RowBox[{"Join", "[", 
         RowBox[{"ansatz", ",", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Subscript", "[", 
              RowBox[{"Rx", ",", " ", "qb"}], "]"}], "[", 
             RowBox[{"my\[Theta]", "[", 
              RowBox[{"c", "++"}], "]"}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"qb", ",", "0", ",", 
              RowBox[{"circuitN", "-", "1"}]}], "}"}]}], "]"}]}], "]"}]}], 
       ";", " ", "\[IndentingNewLine]", 
       RowBox[{"ansatz", "=", 
        RowBox[{"Join", "[", 
         RowBox[{"ansatz", ",", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Subscript", "[", 
              RowBox[{"Rz", ",", " ", "qb"}], "]"}], "[", 
             RowBox[{"my\[Theta]", "[", 
              RowBox[{"c", "++"}], "]"}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"qb", ",", "0", ",", 
              RowBox[{"circuitN", "-", "1"}]}], "}"}]}], "]"}]}], "]"}]}], 
       ";", " ", "\[IndentingNewLine]", 
       RowBox[{"ansatz", "=", 
        RowBox[{"Join", "[", 
         RowBox[{"ansatz", ",", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Subscript", "[", 
              RowBox[{"Ry", ",", " ", "qb"}], "]"}], "[", 
             RowBox[{"my\[Theta]", "[", 
              RowBox[{"c", "++"}], "]"}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"qb", ",", "0", ",", 
              RowBox[{"circuitN", "-", "1"}]}], "}"}]}], "]"}]}], "]"}]}], 
       ";", " ", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"blocks", "=", "1"}], ",", 
         RowBox[{"blocks", "\[LessEqual]", "nblock"}], ",", 
         RowBox[{"blocks", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"ansatz", "=", 
           RowBox[{"Join", "[", 
            RowBox[{"ansatz", ",", 
             RowBox[{"Flatten", "@", 
              RowBox[{"Table", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"R", "[", 
                  RowBox[{
                   RowBox[{"my\[Theta]", "[", 
                    RowBox[{"c", "++"}], "]"}], ",", 
                   RowBox[{
                    SubscriptBox["Z", "qb"], 
                    SubscriptBox["Z", 
                    RowBox[{"qb", "+", "1"}]]}]}], "]"}], 
                 "\[IndentingNewLine]", "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"qb", ",", "0", ",", 
                  RowBox[{"circuitN", "-", "2"}]}], "}"}]}], "]"}]}]}], 
            "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"ansatz", "=", 
           RowBox[{"Join", "[", 
            RowBox[{"ansatz", ",", 
             RowBox[{"Table", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Subscript", "[", 
                 RowBox[{"Rx", ",", " ", "qb"}], "]"}], "[", 
                RowBox[{"my\[Theta]", "[", 
                 RowBox[{"c", "++"}], "]"}], "]"}], ",", 
               RowBox[{"{", 
                RowBox[{"qb", ",", "0", ",", 
                 RowBox[{"circuitN", "-", "1"}]}], "}"}]}], "]"}]}], "]"}]}], 
          ";", "  ", "\[IndentingNewLine]", 
          RowBox[{"ansatz", "=", 
           RowBox[{"Join", "[", 
            RowBox[{"ansatz", ",", 
             RowBox[{"Table", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Subscript", "[", 
                 RowBox[{"Ry", ",", " ", "qb"}], "]"}], "[", 
                RowBox[{"my\[Theta]", "[", 
                 RowBox[{"c", "++"}], "]"}], "]"}], ",", 
               RowBox[{"{", 
                RowBox[{"qb", ",", "0", ",", 
                 RowBox[{"circuitN", "-", "1"}]}], "}"}]}], "]"}]}], "]"}]}], 
          ";"}]}], " ", "\[IndentingNewLine]", "]"}], ";", 
       "\[IndentingNewLine]", " ", 
       RowBox[{"paramCount", "=", 
        RowBox[{"c", "-", "1"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Return", "@", 
        RowBox[{"{", 
         RowBox[{"ansatz", ",", "paramCount"}], "}"}]}], ";"}]}], "\n", 
     "]"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"layers", "=", "3"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ansatz", ",", " ", "numParams"}], "}"}], "=", 
   RowBox[{"generateAnsatzCircuit", "[", 
    RowBox[{"numQs", ",", " ", "layers"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "[", 
   RowBox[{"\"\<Ansatz circuit:\>\"", ",", " ", 
    RowBox[{"DrawCircuit", "[", 
     RowBox[{"ansatz", ",", "numQs"}], "]"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print", "@", "numParams"}], ";"}]}], "Input",
 CellChangeTimes->{{3.8606679172416058`*^9, 3.8606679173846073`*^9}, {
   3.8606679531704717`*^9, 3.8606679533644733`*^9}, {3.8606680970337596`*^9, 
   3.860668118734146*^9}, {3.8606681845004015`*^9, 3.8606682547151546`*^9}, {
   3.860668427577142*^9, 3.860668487236027*^9}, {3.860668542854249*^9, 
   3.860668684613865*^9}, 3.8606688437251887`*^9, {3.86066937480466*^9, 
   3.860669380301657*^9}, {3.86067005899331*^9, 3.860670059038285*^9}, {
   3.860670336351839*^9, 3.8606703363838387`*^9}, {3.860670594504747*^9, 
   3.8606706157483263`*^9}, {3.860670848985903*^9, 3.860670859361286*^9}, {
   3.902919834094934*^9, 3.9029198348314238`*^9}},
 CellLabel->
  "In[292]:=",ExpressionUUID->"3af22d37-ea72-486f-8724-2e969b02ce9d"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Ansatz circuit:\"\>", "\[InvisibleSpace]", 
   GraphicsBox[
    {EdgeForm[GrayLevel[0]], FaceForm[GrayLevel[
     1]], {{LineBox[{{0, 0.5}, {1, 0.5}}], LineBox[{{0, 1.5}, {1, 1.5}}], 
       LineBox[{{0, 2.5}, {1, 2.5}}], LineBox[{{0, 3.5}, {1, 3.5}}], 
       LineBox[{{0, 4.5}, {1, 4.5}}]}, {LineBox[{{1, 0.5}, {2, 0.5}}], 
       LineBox[{{1, 1.5}, {2, 1.5}}], LineBox[{{1, 2.5}, {2, 2.5}}], 
       LineBox[{{1, 3.5}, {2, 3.5}}], LineBox[{{1, 4.5}, {2, 4.5}}]}, {
       LineBox[{{2, 0.5}, {3, 0.5}}], LineBox[{{2, 1.5}, {3, 1.5}}], 
       LineBox[{{2, 2.5}, {3, 2.5}}], LineBox[{{2, 3.5}, {3, 3.5}}], 
       LineBox[{{2, 4.5}, {3, 4.5}}]}, {LineBox[{{3, 0.5}, {4, 0.5}}], 
       LineBox[{{3, 1.5}, {4, 1.5}}], LineBox[{{3, 2.5}, {4, 2.5}}], 
       LineBox[{{3, 3.5}, {4, 3.5}}], LineBox[{{3, 4.5}, {4, 4.5}}]}, {
       LineBox[{{4, 0.5}, {5, 0.5}}], LineBox[{{4, 1.5}, {5, 1.5}}], 
       LineBox[{{4, 2.5}, {5, 2.5}}], LineBox[{{4, 3.5}, {5, 3.5}}], 
       LineBox[{{4, 4.5}, {5, 4.5}}]}, {LineBox[{{5, 0.5}, {6, 0.5}}], 
       LineBox[{{5, 1.5}, {6, 1.5}}], LineBox[{{5, 2.5}, {6, 2.5}}], 
       LineBox[{{5, 3.5}, {6, 3.5}}], LineBox[{{5, 4.5}, {6, 4.5}}]}, {
       LineBox[{{6, 0.5}, {7, 0.5}}], LineBox[{{6, 1.5}, {7, 1.5}}], 
       LineBox[{{6, 2.5}, {7, 2.5}}], LineBox[{{6, 3.5}, {7, 3.5}}], 
       LineBox[{{6, 4.5}, {7, 4.5}}]}, {LineBox[{{7, 0.5}, {8, 0.5}}], 
       LineBox[{{7, 1.5}, {8, 1.5}}], LineBox[{{7, 2.5}, {8, 2.5}}], 
       LineBox[{{7, 3.5}, {8, 3.5}}], LineBox[{{7, 4.5}, {8, 4.5}}]}, {
       LineBox[{{8, 0.5}, {9, 0.5}}], LineBox[{{8, 1.5}, {9, 1.5}}], 
       LineBox[{{8, 2.5}, {9, 2.5}}], LineBox[{{8, 3.5}, {9, 3.5}}], 
       LineBox[{{8, 4.5}, {9, 4.5}}]}, {LineBox[{{9, 0.5}, {10, 0.5}}], 
       LineBox[{{9, 1.5}, {10, 1.5}}], LineBox[{{9, 2.5}, {10, 2.5}}], 
       LineBox[{{9, 3.5}, {10, 3.5}}], LineBox[{{9, 4.5}, {10, 4.5}}]}, {
       LineBox[{{10, 0.5}, {11, 0.5}}], LineBox[{{10, 1.5}, {11, 1.5}}], 
       LineBox[{{10, 2.5}, {11, 2.5}}], LineBox[{{10, 3.5}, {11, 3.5}}], 
       LineBox[{{10, 4.5}, {11, 4.5}}]}, {LineBox[{{11, 0.5}, {12, 0.5}}], 
       LineBox[{{11, 1.5}, {12, 1.5}}], LineBox[{{11, 2.5}, {12, 2.5}}], 
       LineBox[{{11, 3.5}, {12, 3.5}}], LineBox[{{11, 4.5}, {12, 4.5}}]}, {
       LineBox[{{12, 0.5}, {13, 0.5}}], LineBox[{{12, 1.5}, {13, 1.5}}], 
       LineBox[{{12, 2.5}, {13, 2.5}}], LineBox[{{12, 3.5}, {13, 3.5}}], 
       LineBox[{{12, 4.5}, {13, 4.5}}]}, {LineBox[{{13, 0.5}, {14, 0.5}}], 
       LineBox[{{13, 1.5}, {14, 1.5}}], LineBox[{{13, 2.5}, {14, 2.5}}], 
       LineBox[{{13, 3.5}, {14, 3.5}}], LineBox[{{13, 4.5}, {14, 4.5}}]}, {
       LineBox[{{14, 0.5}, {15, 0.5}}], LineBox[{{14, 1.5}, {15, 1.5}}], 
       LineBox[{{14, 2.5}, {15, 2.5}}], LineBox[{{14, 3.5}, {15, 3.5}}], 
       LineBox[{{14, 4.5}, {15, 4.5}}]}, {LineBox[{{15, 0.5}, {16, 0.5}}], 
       LineBox[{{15, 1.5}, {16, 1.5}}], LineBox[{{15, 2.5}, {16, 2.5}}], 
       LineBox[{{15, 3.5}, {16, 3.5}}], LineBox[{{15, 4.5}, {16, 4.5}}]}, {
       LineBox[{{16, 0.5}, {17, 0.5}}], LineBox[{{16, 1.5}, {17, 1.5}}], 
       LineBox[{{16, 2.5}, {17, 2.5}}], LineBox[{{16, 3.5}, {17, 3.5}}], 
       LineBox[{{16, 4.5}, {17, 4.5}}]}}, {{
       RectangleBox[{0.1, 0.1}, {0.9, 0.9}], 
       InsetBox["\<\"Rx\"\>", {0.5, 0.5}]}, {
       RectangleBox[{0.1, 1.1}, {0.9, 1.9}], 
       InsetBox["\<\"Rx\"\>", {0.5, 1.5}]}, {
       RectangleBox[{0.1, 2.1}, {0.9, 2.9}], 
       InsetBox["\<\"Rx\"\>", {0.5, 2.5}]}, {
       RectangleBox[{0.1, 3.1}, {0.9, 3.9}], 
       InsetBox["\<\"Rx\"\>", {0.5, 3.5}]}, {
       RectangleBox[{0.1, 4.1}, {0.9, 4.9}], 
       InsetBox["\<\"Rx\"\>", {0.5, 4.5}]}, {
       RectangleBox[{1.1, 0.1}, {1.9, 0.9}], 
       InsetBox["\<\"Rz\"\>", {1.5, 0.5}]}, {
       RectangleBox[{1.1, 1.1}, {1.9, 1.9}], 
       InsetBox["\<\"Rz\"\>", {1.5, 1.5}]}, {
       RectangleBox[{1.1, 2.1}, {1.9, 2.9}], 
       InsetBox["\<\"Rz\"\>", {1.5, 2.5}]}, {
       RectangleBox[{1.1, 3.1}, {1.9, 3.9}], 
       InsetBox["\<\"Rz\"\>", {1.5, 3.5}]}, {
       RectangleBox[{1.1, 4.1}, {1.9, 4.9}], 
       InsetBox["\<\"Rz\"\>", {1.5, 4.5}]}, {
       RectangleBox[{2.1, 0.1}, {2.9, 0.9}], 
       InsetBox["\<\"Ry\"\>", {2.5, 0.5}]}, {
       RectangleBox[{2.1, 1.1}, {2.9, 1.9}], 
       InsetBox["\<\"Ry\"\>", {2.5, 1.5}]}, {
       RectangleBox[{2.1, 2.1}, {2.9, 2.9}], 
       InsetBox["\<\"Ry\"\>", {2.5, 2.5}]}, {
       RectangleBox[{2.1, 3.1}, {2.9, 3.9}], 
       InsetBox["\<\"Ry\"\>", {2.5, 3.5}]}, {
       RectangleBox[{2.1, 4.1}, {2.9, 4.9}], 
       InsetBox["\<\"Ry\"\>", {2.5, 4.5}]}, {
       LineBox[{{3.5, 0.5}, {3.5, 1.5}}], {
        RectangleBox[{3.1, 0.1}, {3.9, 0.9}], 
        InsetBox["\<\"Rz\"\>", {3.5, 0.5}]}, {
        RectangleBox[{3.1, 1.1}, {3.9, 1.9}], 
        InsetBox["\<\"Rz\"\>", {3.5, 1.5}]}}, {
       LineBox[{{4.5, 1.5}, {4.5, 2.5}}], {
        RectangleBox[{4.1, 1.1}, {4.9, 1.9}], 
        InsetBox["\<\"Rz\"\>", {4.5, 1.5}]}, {
        RectangleBox[{4.1, 2.1}, {4.9, 2.9}], 
        InsetBox["\<\"Rz\"\>", {4.5, 2.5}]}}, {
       RectangleBox[{4.1, 0.1}, {4.9, 0.9}], 
       InsetBox["\<\"Rx\"\>", {4.5, 0.5}]}, {
       LineBox[{{5.5, 2.5}, {5.5, 3.5}}], {
        RectangleBox[{5.1, 2.1}, {5.9, 2.9}], 
        InsetBox["\<\"Rz\"\>", {5.5, 2.5}]}, {
        RectangleBox[{5.1, 3.1}, {5.9, 3.9}], 
        InsetBox["\<\"Rz\"\>", {5.5, 3.5}]}}, {
       RectangleBox[{5.1, 1.1}, {5.9, 1.9}], 
       InsetBox["\<\"Rx\"\>", {5.5, 1.5}]}, {
       RectangleBox[{5.1, 0.1}, {5.9, 0.9}], 
       InsetBox["\<\"Ry\"\>", {5.5, 0.5}]}, {
       LineBox[{{6.5, 3.5}, {6.5, 4.5}}], {
        RectangleBox[{6.1, 3.1}, {6.9, 3.9}], 
        InsetBox["\<\"Rz\"\>", {6.5, 3.5}]}, {
        RectangleBox[{6.1, 4.1}, {6.9, 4.9}], 
        InsetBox["\<\"Rz\"\>", {6.5, 4.5}]}}, {
       RectangleBox[{6.1, 2.1}, {6.9, 2.9}], 
       InsetBox["\<\"Rx\"\>", {6.5, 2.5}]}, {
       RectangleBox[{6.1, 1.1}, {6.9, 1.9}], 
       InsetBox["\<\"Ry\"\>", {6.5, 1.5}]}, {
       RectangleBox[{7.1, 3.1}, {7.9, 3.9}], 
       InsetBox["\<\"Rx\"\>", {7.5, 3.5}]}, {
       RectangleBox[{7.1, 4.1}, {7.9, 4.9}], 
       InsetBox["\<\"Rx\"\>", {7.5, 4.5}]}, {
       RectangleBox[{7.1, 2.1}, {7.9, 2.9}], 
       InsetBox["\<\"Ry\"\>", {7.5, 2.5}]}, {
       LineBox[{{7.5, 0.5}, {7.5, 1.5}}], {
        RectangleBox[{7.1, 0.1}, {7.9, 0.9}], 
        InsetBox["\<\"Rz\"\>", {7.5, 0.5}]}, {
        RectangleBox[{7.1, 1.1}, {7.9, 1.9}], 
        InsetBox["\<\"Rz\"\>", {7.5, 1.5}]}}, {
       RectangleBox[{8.1, 3.1}, {8.9, 3.9}], 
       InsetBox["\<\"Ry\"\>", {8.5, 3.5}]}, {
       RectangleBox[{8.1, 4.1}, {8.9, 4.9}], 
       InsetBox["\<\"Ry\"\>", {8.5, 4.5}]}, {
       LineBox[{{8.5, 1.5}, {8.5, 2.5}}], {
        RectangleBox[{8.1, 1.1}, {8.9, 1.9}], 
        InsetBox["\<\"Rz\"\>", {8.5, 1.5}]}, {
        RectangleBox[{8.1, 2.1}, {8.9, 2.9}], 
        InsetBox["\<\"Rz\"\>", {8.5, 2.5}]}}, {
       RectangleBox[{8.1, 0.1}, {8.9, 0.9}], 
       InsetBox["\<\"Rx\"\>", {8.5, 0.5}]}, {
       LineBox[{{9.5, 2.5}, {9.5, 3.5}}], {
        RectangleBox[{9.1, 2.1}, {9.9, 2.9}], 
        InsetBox["\<\"Rz\"\>", {9.5, 2.5}]}, {
        RectangleBox[{9.1, 3.1}, {9.9, 3.9}], 
        InsetBox["\<\"Rz\"\>", {9.5, 3.5}]}}, {
       RectangleBox[{9.1, 1.1}, {9.9, 1.9}], 
       InsetBox["\<\"Rx\"\>", {9.5, 1.5}]}, {
       RectangleBox[{9.1, 0.1}, {9.9, 0.9}], 
       InsetBox["\<\"Ry\"\>", {9.5, 0.5}]}, {
       LineBox[{{10.5, 3.5}, {10.5, 4.5}}], {
        RectangleBox[{10.1, 3.1}, {10.9, 3.9}], 
        InsetBox["\<\"Rz\"\>", {10.5, 3.5}]}, {
        RectangleBox[{10.1, 4.1}, {10.9, 4.9}], 
        InsetBox["\<\"Rz\"\>", {10.5, 4.5}]}}, {
       RectangleBox[{10.1, 2.1}, {10.9, 2.9}], 
       InsetBox["\<\"Rx\"\>", {10.5, 2.5}]}, {
       RectangleBox[{10.1, 1.1}, {10.9, 1.9}], 
       InsetBox["\<\"Ry\"\>", {10.5, 1.5}]}, {
       RectangleBox[{11.1, 3.1}, {11.9, 3.9}], 
       InsetBox["\<\"Rx\"\>", {11.5, 3.5}]}, {
       RectangleBox[{11.1, 4.1}, {11.9, 4.9}], 
       InsetBox["\<\"Rx\"\>", {11.5, 4.5}]}, {
       RectangleBox[{11.1, 2.1}, {11.9, 2.9}], 
       InsetBox["\<\"Ry\"\>", {11.5, 2.5}]}, {
       LineBox[{{11.5, 0.5}, {11.5, 1.5}}], {
        RectangleBox[{11.1, 0.1}, {11.9, 0.9}], 
        InsetBox["\<\"Rz\"\>", {11.5, 0.5}]}, {
        RectangleBox[{11.1, 1.1}, {11.9, 1.9}], 
        InsetBox["\<\"Rz\"\>", {11.5, 1.5}]}}, {
       RectangleBox[{12.1, 3.1}, {12.9, 3.9}], 
       InsetBox["\<\"Ry\"\>", {12.5, 3.5}]}, {
       RectangleBox[{12.1, 4.1}, {12.9, 4.9}], 
       InsetBox["\<\"Ry\"\>", {12.5, 4.5}]}, {
       LineBox[{{12.5, 1.5}, {12.5, 2.5}}], {
        RectangleBox[{12.1, 1.1}, {12.9, 1.9}], 
        InsetBox["\<\"Rz\"\>", {12.5, 1.5}]}, {
        RectangleBox[{12.1, 2.1}, {12.9, 2.9}], 
        InsetBox["\<\"Rz\"\>", {12.5, 2.5}]}}, {
       RectangleBox[{12.1, 0.1}, {12.9, 0.9}], 
       InsetBox["\<\"Rx\"\>", {12.5, 0.5}]}, {
       LineBox[{{13.5, 2.5}, {13.5, 3.5}}], {
        RectangleBox[{13.1, 2.1}, {13.9, 2.9}], 
        InsetBox["\<\"Rz\"\>", {13.5, 2.5}]}, {
        RectangleBox[{13.1, 3.1}, {13.9, 3.9}], 
        InsetBox["\<\"Rz\"\>", {13.5, 3.5}]}}, {
       RectangleBox[{13.1, 1.1}, {13.9, 1.9}], 
       InsetBox["\<\"Rx\"\>", {13.5, 1.5}]}, {
       RectangleBox[{13.1, 0.1}, {13.9, 0.9}], 
       InsetBox["\<\"Ry\"\>", {13.5, 0.5}]}, {
       LineBox[{{14.5, 3.5}, {14.5, 4.5}}], {
        RectangleBox[{14.1, 3.1}, {14.9, 3.9}], 
        InsetBox["\<\"Rz\"\>", {14.5, 3.5}]}, {
        RectangleBox[{14.1, 4.1}, {14.9, 4.9}], 
        InsetBox["\<\"Rz\"\>", {14.5, 4.5}]}}, {
       RectangleBox[{14.1, 2.1}, {14.9, 2.9}], 
       InsetBox["\<\"Rx\"\>", {14.5, 2.5}]}, {
       RectangleBox[{14.1, 1.1}, {14.9, 1.9}], 
       InsetBox["\<\"Ry\"\>", {14.5, 1.5}]}, {
       RectangleBox[{15.1, 3.1}, {15.9, 3.9}], 
       InsetBox["\<\"Rx\"\>", {15.5, 3.5}]}, {
       RectangleBox[{15.1, 4.1}, {15.9, 4.9}], 
       InsetBox["\<\"Rx\"\>", {15.5, 4.5}]}, {
       RectangleBox[{15.1, 2.1}, {15.9, 2.9}], 
       InsetBox["\<\"Ry\"\>", {15.5, 2.5}]}, {
       RectangleBox[{16.1, 3.1}, {16.9, 3.9}], 
       InsetBox["\<\"Ry\"\>", {16.5, 3.5}]}, {
       RectangleBox[{16.1, 4.1}, {16.9, 4.9}], 
       InsetBox["\<\"Ry\"\>", {16.5, 4.5}]}}},
    ImageSize->540,
    PlotRangePadding->None]}],
  SequenceForm["Ansatz circuit:", 
   Graphics[{
     FaceForm[
      GrayLevel[1]], 
     EdgeForm[
      GrayLevel[0]], {{
       Line[{{0, 0.5}, {1, 0.5}}], 
       Line[{{0, 1.5}, {1, 1.5}}], 
       Line[{{0, 2.5}, {1, 2.5}}], 
       Line[{{0, 3.5}, {1, 3.5}}], 
       Line[{{0, 4.5}, {1, 4.5}}]}, {
       Line[{{1, 0.5}, {2, 0.5}}], 
       Line[{{1, 1.5}, {2, 1.5}}], 
       Line[{{1, 2.5}, {2, 2.5}}], 
       Line[{{1, 3.5}, {2, 3.5}}], 
       Line[{{1, 4.5}, {2, 4.5}}]}, {
       Line[{{2, 0.5}, {3, 0.5}}], 
       Line[{{2, 1.5}, {3, 1.5}}], 
       Line[{{2, 2.5}, {3, 2.5}}], 
       Line[{{2, 3.5}, {3, 3.5}}], 
       Line[{{2, 4.5}, {3, 4.5}}]}, {
       Line[{{3, 0.5}, {4, 0.5}}], 
       Line[{{3, 1.5}, {4, 1.5}}], 
       Line[{{3, 2.5}, {4, 2.5}}], 
       Line[{{3, 3.5}, {4, 3.5}}], 
       Line[{{3, 4.5}, {4, 4.5}}]}, {
       Line[{{4, 0.5}, {5, 0.5}}], 
       Line[{{4, 1.5}, {5, 1.5}}], 
       Line[{{4, 2.5}, {5, 2.5}}], 
       Line[{{4, 3.5}, {5, 3.5}}], 
       Line[{{4, 4.5}, {5, 4.5}}]}, {
       Line[{{5, 0.5}, {6, 0.5}}], 
       Line[{{5, 1.5}, {6, 1.5}}], 
       Line[{{5, 2.5}, {6, 2.5}}], 
       Line[{{5, 3.5}, {6, 3.5}}], 
       Line[{{5, 4.5}, {6, 4.5}}]}, {
       Line[{{6, 0.5}, {7, 0.5}}], 
       Line[{{6, 1.5}, {7, 1.5}}], 
       Line[{{6, 2.5}, {7, 2.5}}], 
       Line[{{6, 3.5}, {7, 3.5}}], 
       Line[{{6, 4.5}, {7, 4.5}}]}, {
       Line[{{7, 0.5}, {8, 0.5}}], 
       Line[{{7, 1.5}, {8, 1.5}}], 
       Line[{{7, 2.5}, {8, 2.5}}], 
       Line[{{7, 3.5}, {8, 3.5}}], 
       Line[{{7, 4.5}, {8, 4.5}}]}, {
       Line[{{8, 0.5}, {9, 0.5}}], 
       Line[{{8, 1.5}, {9, 1.5}}], 
       Line[{{8, 2.5}, {9, 2.5}}], 
       Line[{{8, 3.5}, {9, 3.5}}], 
       Line[{{8, 4.5}, {9, 4.5}}]}, {
       Line[{{9, 0.5}, {10, 0.5}}], 
       Line[{{9, 1.5}, {10, 1.5}}], 
       Line[{{9, 2.5}, {10, 2.5}}], 
       Line[{{9, 3.5}, {10, 3.5}}], 
       Line[{{9, 4.5}, {10, 4.5}}]}, {
       Line[{{10, 0.5}, {11, 0.5}}], 
       Line[{{10, 1.5}, {11, 1.5}}], 
       Line[{{10, 2.5}, {11, 2.5}}], 
       Line[{{10, 3.5}, {11, 3.5}}], 
       Line[{{10, 4.5}, {11, 4.5}}]}, {
       Line[{{11, 0.5}, {12, 0.5}}], 
       Line[{{11, 1.5}, {12, 1.5}}], 
       Line[{{11, 2.5}, {12, 2.5}}], 
       Line[{{11, 3.5}, {12, 3.5}}], 
       Line[{{11, 4.5}, {12, 4.5}}]}, {
       Line[{{12, 0.5}, {13, 0.5}}], 
       Line[{{12, 1.5}, {13, 1.5}}], 
       Line[{{12, 2.5}, {13, 2.5}}], 
       Line[{{12, 3.5}, {13, 3.5}}], 
       Line[{{12, 4.5}, {13, 4.5}}]}, {
       Line[{{13, 0.5}, {14, 0.5}}], 
       Line[{{13, 1.5}, {14, 1.5}}], 
       Line[{{13, 2.5}, {14, 2.5}}], 
       Line[{{13, 3.5}, {14, 3.5}}], 
       Line[{{13, 4.5}, {14, 4.5}}]}, {
       Line[{{14, 0.5}, {15, 0.5}}], 
       Line[{{14, 1.5}, {15, 1.5}}], 
       Line[{{14, 2.5}, {15, 2.5}}], 
       Line[{{14, 3.5}, {15, 3.5}}], 
       Line[{{14, 4.5}, {15, 4.5}}]}, {
       Line[{{15, 0.5}, {16, 0.5}}], 
       Line[{{15, 1.5}, {16, 1.5}}], 
       Line[{{15, 2.5}, {16, 2.5}}], 
       Line[{{15, 3.5}, {16, 3.5}}], 
       Line[{{15, 4.5}, {16, 4.5}}]}, {
       Line[{{16, 0.5}, {17, 0.5}}], 
       Line[{{16, 1.5}, {17, 1.5}}], 
       Line[{{16, 2.5}, {17, 2.5}}], 
       Line[{{16, 3.5}, {17, 3.5}}], 
       Line[{{16, 4.5}, {17, 4.5}}]}}, {{
       Rectangle[{0.1, 0.1}, {0.9, 0.9}], 
       Text["Rx", {0.5, 0.5}]}, {
       Rectangle[{0.1, 1.1}, {0.9, 1.9}], 
       Text["Rx", {0.5, 1.5}]}, {
       Rectangle[{0.1, 2.1}, {0.9, 2.9}], 
       Text["Rx", {0.5, 2.5}]}, {
       Rectangle[{0.1, 3.1}, {0.9, 3.9}], 
       Text["Rx", {0.5, 3.5}]}, {
       Rectangle[{0.1, 4.1}, {0.9, 4.9}], 
       Text["Rx", {0.5, 4.5}]}, {
       Rectangle[{1.1, 0.1}, {1.9, 0.9}], 
       Text["Rz", {1.5, 0.5}]}, {
       Rectangle[{1.1, 1.1}, {1.9, 1.9}], 
       Text["Rz", {1.5, 1.5}]}, {
       Rectangle[{1.1, 2.1}, {1.9, 2.9}], 
       Text["Rz", {1.5, 2.5}]}, {
       Rectangle[{1.1, 3.1}, {1.9, 3.9}], 
       Text["Rz", {1.5, 3.5}]}, {
       Rectangle[{1.1, 4.1}, {1.9, 4.9}], 
       Text["Rz", {1.5, 4.5}]}, {
       Rectangle[{2.1, 0.1}, {2.9, 0.9}], 
       Text["Ry", {2.5, 0.5}]}, {
       Rectangle[{2.1, 1.1}, {2.9, 1.9}], 
       Text["Ry", {2.5, 1.5}]}, {
       Rectangle[{2.1, 2.1}, {2.9, 2.9}], 
       Text["Ry", {2.5, 2.5}]}, {
       Rectangle[{2.1, 3.1}, {2.9, 3.9}], 
       Text["Ry", {2.5, 3.5}]}, {
       Rectangle[{2.1, 4.1}, {2.9, 4.9}], 
       Text["Ry", {2.5, 4.5}]}, {
       Line[{{3.5, 0.5}, {3.5, 1.5}}], {
        Rectangle[{3.1, 0.1}, {3.9, 0.9}], 
        Text["Rz", {3.5, 0.5}]}, {
        Rectangle[{3.1, 1.1}, {3.9, 1.9}], 
        Text["Rz", {3.5, 1.5}]}}, {
       Line[{{4.5, 1.5}, {4.5, 2.5}}], {
        Rectangle[{4.1, 1.1}, {4.9, 1.9}], 
        Text["Rz", {4.5, 1.5}]}, {
        Rectangle[{4.1, 2.1}, {4.9, 2.9}], 
        Text["Rz", {4.5, 2.5}]}}, {
       Rectangle[{4.1, 0.1}, {4.9, 0.9}], 
       Text["Rx", {4.5, 0.5}]}, {
       Line[{{5.5, 2.5}, {5.5, 3.5}}], {
        Rectangle[{5.1, 2.1}, {5.9, 2.9}], 
        Text["Rz", {5.5, 2.5}]}, {
        Rectangle[{5.1, 3.1}, {5.9, 3.9}], 
        Text["Rz", {5.5, 3.5}]}}, {
       Rectangle[{5.1, 1.1}, {5.9, 1.9}], 
       Text["Rx", {5.5, 1.5}]}, {
       Rectangle[{5.1, 0.1}, {5.9, 0.9}], 
       Text["Ry", {5.5, 0.5}]}, {
       Line[{{6.5, 3.5}, {6.5, 4.5}}], {
        Rectangle[{6.1, 3.1}, {6.9, 3.9}], 
        Text["Rz", {6.5, 3.5}]}, {
        Rectangle[{6.1, 4.1}, {6.9, 4.9}], 
        Text["Rz", {6.5, 4.5}]}}, {
       Rectangle[{6.1, 2.1}, {6.9, 2.9}], 
       Text["Rx", {6.5, 2.5}]}, {
       Rectangle[{6.1, 1.1}, {6.9, 1.9}], 
       Text["Ry", {6.5, 1.5}]}, {
       Rectangle[{7.1, 3.1}, {7.9, 3.9}], 
       Text["Rx", {7.5, 3.5}]}, {
       Rectangle[{7.1, 4.1}, {7.9, 4.9}], 
       Text["Rx", {7.5, 4.5}]}, {
       Rectangle[{7.1, 2.1}, {7.9, 2.9}], 
       Text["Ry", {7.5, 2.5}]}, {
       Line[{{7.5, 0.5}, {7.5, 1.5}}], {
        Rectangle[{7.1, 0.1}, {7.9, 0.9}], 
        Text["Rz", {7.5, 0.5}]}, {
        Rectangle[{7.1, 1.1}, {7.9, 1.9}], 
        Text["Rz", {7.5, 1.5}]}}, {
       Rectangle[{8.1, 3.1}, {8.9, 3.9}], 
       Text["Ry", {8.5, 3.5}]}, {
       Rectangle[{8.1, 4.1}, {8.9, 4.9}], 
       Text["Ry", {8.5, 4.5}]}, {
       Line[{{8.5, 1.5}, {8.5, 2.5}}], {
        Rectangle[{8.1, 1.1}, {8.9, 1.9}], 
        Text["Rz", {8.5, 1.5}]}, {
        Rectangle[{8.1, 2.1}, {8.9, 2.9}], 
        Text["Rz", {8.5, 2.5}]}}, {
       Rectangle[{8.1, 0.1}, {8.9, 0.9}], 
       Text["Rx", {8.5, 0.5}]}, {
       Line[{{9.5, 2.5}, {9.5, 3.5}}], {
        Rectangle[{9.1, 2.1}, {9.9, 2.9}], 
        Text["Rz", {9.5, 2.5}]}, {
        Rectangle[{9.1, 3.1}, {9.9, 3.9}], 
        Text["Rz", {9.5, 3.5}]}}, {
       Rectangle[{9.1, 1.1}, {9.9, 1.9}], 
       Text["Rx", {9.5, 1.5}]}, {
       Rectangle[{9.1, 0.1}, {9.9, 0.9}], 
       Text["Ry", {9.5, 0.5}]}, {
       Line[{{10.5, 3.5}, {10.5, 4.5}}], {
        Rectangle[{10.1, 3.1}, {10.9, 3.9}], 
        Text["Rz", {10.5, 3.5}]}, {
        Rectangle[{10.1, 4.1}, {10.9, 4.9}], 
        Text["Rz", {10.5, 4.5}]}}, {
       Rectangle[{10.1, 2.1}, {10.9, 2.9}], 
       Text["Rx", {10.5, 2.5}]}, {
       Rectangle[{10.1, 1.1}, {10.9, 1.9}], 
       Text["Ry", {10.5, 1.5}]}, {
       Rectangle[{11.1, 3.1}, {11.9, 3.9}], 
       Text["Rx", {11.5, 3.5}]}, {
       Rectangle[{11.1, 4.1}, {11.9, 4.9}], 
       Text["Rx", {11.5, 4.5}]}, {
       Rectangle[{11.1, 2.1}, {11.9, 2.9}], 
       Text["Ry", {11.5, 2.5}]}, {
       Line[{{11.5, 0.5}, {11.5, 1.5}}], {
        Rectangle[{11.1, 0.1}, {11.9, 0.9}], 
        Text["Rz", {11.5, 0.5}]}, {
        Rectangle[{11.1, 1.1}, {11.9, 1.9}], 
        Text["Rz", {11.5, 1.5}]}}, {
       Rectangle[{12.1, 3.1}, {12.9, 3.9}], 
       Text["Ry", {12.5, 3.5}]}, {
       Rectangle[{12.1, 4.1}, {12.9, 4.9}], 
       Text["Ry", {12.5, 4.5}]}, {
       Line[{{12.5, 1.5}, {12.5, 2.5}}], {
        Rectangle[{12.1, 1.1}, {12.9, 1.9}], 
        Text["Rz", {12.5, 1.5}]}, {
        Rectangle[{12.1, 2.1}, {12.9, 2.9}], 
        Text["Rz", {12.5, 2.5}]}}, {
       Rectangle[{12.1, 0.1}, {12.9, 0.9}], 
       Text["Rx", {12.5, 0.5}]}, {
       Line[{{13.5, 2.5}, {13.5, 3.5}}], {
        Rectangle[{13.1, 2.1}, {13.9, 2.9}], 
        Text["Rz", {13.5, 2.5}]}, {
        Rectangle[{13.1, 3.1}, {13.9, 3.9}], 
        Text["Rz", {13.5, 3.5}]}}, {
       Rectangle[{13.1, 1.1}, {13.9, 1.9}], 
       Text["Rx", {13.5, 1.5}]}, {
       Rectangle[{13.1, 0.1}, {13.9, 0.9}], 
       Text["Ry", {13.5, 0.5}]}, {
       Line[{{14.5, 3.5}, {14.5, 4.5}}], {
        Rectangle[{14.1, 3.1}, {14.9, 3.9}], 
        Text["Rz", {14.5, 3.5}]}, {
        Rectangle[{14.1, 4.1}, {14.9, 4.9}], 
        Text["Rz", {14.5, 4.5}]}}, {
       Rectangle[{14.1, 2.1}, {14.9, 2.9}], 
       Text["Rx", {14.5, 2.5}]}, {
       Rectangle[{14.1, 1.1}, {14.9, 1.9}], 
       Text["Ry", {14.5, 1.5}]}, {
       Rectangle[{15.1, 3.1}, {15.9, 3.9}], 
       Text["Rx", {15.5, 3.5}]}, {
       Rectangle[{15.1, 4.1}, {15.9, 4.9}], 
       Text["Rx", {15.5, 4.5}]}, {
       Rectangle[{15.1, 2.1}, {15.9, 2.9}], 
       Text["Ry", {15.5, 2.5}]}, {
       Rectangle[{16.1, 3.1}, {16.9, 3.9}], 
       Text["Ry", {16.5, 3.5}]}, {
       Rectangle[{16.1, 4.1}, {16.9, 4.9}], 
       Text["Ry", {16.5, 4.5}]}}}, {}, ImageSize -> 540, PlotRangePadding -> 
    None]],
  Editable->False]], "Print",
 CellChangeTimes->{{3.860668661291054*^9, 3.8606686851219006`*^9}, 
   3.8606693806116915`*^9, 3.860669837124187*^9, 3.8606700632342772`*^9, {
   3.8606701274690266`*^9, 3.8606701458226857`*^9}, 3.860670198945935*^9, 
   3.8606702486618395`*^9, {3.8606703090269794`*^9, 3.860670340556077*^9}, 
   3.860670439121554*^9, 3.8606704700099516`*^9, 3.860670508133952*^9, 
   3.8606706214223213`*^9, 3.8606706624173203`*^9, 3.8606707791238203`*^9, 
   3.8606708641303167`*^9, 3.86067111170068*^9, 3.8606711678893056`*^9, 
   3.8606718527087445`*^9, 3.8606719209693832`*^9, {3.860671973179383*^9, 
   3.860671983568384*^9}, 3.8606720678062983`*^9, 3.902919817119482*^9, 
   3.9029198763654633`*^9, 3.9029199932054396`*^9, 3.9029200561477275`*^9, 
   3.902920118597128*^9, 3.902920191414117*^9, {3.9029208313598776`*^9, 
   3.902920843427485*^9}, 3.902920906057064*^9},
 CellLabel->
  "During evaluation of \
In[292]:=",ExpressionUUID->"454199c5-45af-41eb-8639-299cd6ac9b4e"],

Cell[BoxData["57"], "Print",
 CellChangeTimes->{{3.860668661291054*^9, 3.8606686851219006`*^9}, 
   3.8606693806116915`*^9, 3.860669837124187*^9, 3.8606700632342772`*^9, {
   3.8606701274690266`*^9, 3.8606701458226857`*^9}, 3.860670198945935*^9, 
   3.8606702486618395`*^9, {3.8606703090269794`*^9, 3.860670340556077*^9}, 
   3.860670439121554*^9, 3.8606704700099516`*^9, 3.860670508133952*^9, 
   3.8606706214223213`*^9, 3.8606706624173203`*^9, 3.8606707791238203`*^9, 
   3.8606708641303167`*^9, 3.86067111170068*^9, 3.8606711678893056`*^9, 
   3.8606718527087445`*^9, 3.8606719209693832`*^9, {3.860671973179383*^9, 
   3.860671983568384*^9}, 3.8606720678062983`*^9, 3.902919817119482*^9, 
   3.9029198763654633`*^9, 3.9029199932054396`*^9, 3.9029200561477275`*^9, 
   3.902920118597128*^9, 3.902920191414117*^9, {3.9029208313598776`*^9, 
   3.902920843427485*^9}, 3.902920906072062*^9},
 CellLabel->
  "During evaluation of \
In[292]:=",ExpressionUUID->"19b8dbac-7d12-4a69-af2e-58eaec6eaa1c"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "For the root finding method, we calculate covariance functions by defining \
an operator pool ",
 Cell[BoxData[
  FormBox[
   TemplateBox[<|"boxes" -> FormBox[
       SubscriptBox[
        StyleBox["O", "TI"], 
        StyleBox["k", "TI"]], TraditionalForm], "errors" -> {}, "input" -> 
     "O_k", "state" -> "Boxes"|>,
    "TeXAssistantTemplate"], TraditionalForm]],ExpressionUUID->
  "d2f31a0e-eb61-41c8-a342-8bc4d90fb7ea"],
 " and calculating\n",
 Cell[BoxData[
  FormBox[
   TemplateBox[<|"boxes" -> FormBox[
       RowBox[{
         SubscriptBox[
          StyleBox["f", "TI"], 
          StyleBox["k", "TI"]], 
         RowBox[{"(", "\[Theta]", ")"}], "\[LongEqual]", "<", 
         StyleBox["H", "TI"], ",", 
         SubscriptBox[
          StyleBox["O", "TI"], 
          StyleBox["k", "TI"]], 
         SubscriptBox[">", "\[Psi]"], "\[LongEqual]", "<", "\[Psi]", "|", 
         StyleBox["H", "TI"], 
         SubscriptBox[
          StyleBox["O", "TI"], 
          StyleBox["k", "TI"]], "|", "\[Psi]", ">", "-", "<", "\[Psi]", "|", 
         SubscriptBox[
          StyleBox["O", "TI"], 
          StyleBox["k", "TI"]], "|", "\[Psi]", ">", "<", "\[Psi]", "|", 
         StyleBox["H", "TI"], "|", "\[Psi]", ">"}], TraditionalForm], 
     "errors" -> {}, "input" -> 
     "f_k(\\theta)=< H, O_k >_\\psi = <\\psi|H O_k|\\psi> - \
<\\psi|O_k|\\psi><\\psi|H |\\psi>", "state" -> "Boxes"|>,
    "TeXAssistantTemplate"], TraditionalForm]],ExpressionUUID->
  "575c70bc-128a-420b-a4ce-f7a32d913bae"],
 " and the Jacobian\n",
 Cell[BoxData[
  FormBox[
   TemplateBox[<|"boxes" -> FormBox[
       RowBox[{
         SubscriptBox[
          StyleBox["J", "TI"], 
          RowBox[{
            StyleBox["j", "TI"], 
            StyleBox["k", "TI"]}]], 
         RowBox[{"(", "\[Theta]", ")"}], "\[LongEqual]", 
         FractionBox[
          RowBox[{"\[PartialD]", 
            SubscriptBox[
             StyleBox["f", "TI"], 
             StyleBox["k", "TI"]]}], 
          RowBox[{"\[PartialD]", 
            SubscriptBox["\[Theta]", 
             StyleBox["j", "TI"]]}]]}], TraditionalForm], "errors" -> {}, 
     "input" -> 
     "J_{jk}(\\theta)  = \\frac{\\partial f_k}{\\partial \\theta_j}", "state" -> 
     "Boxes"|>,
    "TeXAssistantTemplate"], TraditionalForm]],ExpressionUUID->
  "09f9838a-1070-476b-8ac2-6179a14c0234"],
 "\nThe operator pool we choose here is all 3-local Pauli strings\n"
}], "Text",
 CellChangeTimes->{{3.8606689715479307`*^9, 3.860669045307125*^9}, {
   3.8606692266391315`*^9, 3.8606692334587746`*^9}, {3.860669303884713*^9, 
   3.8606693180466795`*^9}, 3.9029196854784737`*^9, {3.902920968144115*^9, 
   3.9029209716376414`*^9}},ExpressionUUID->"caff12e3-ad50-48cc-8b36-\
c8b8643e36ed"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"pauli", "[", 
     RowBox[{"type_", ",", " ", "site_"}], "]"}], ":=", 
    SubscriptBox["type", "site"]}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"getListOf3LocalPaulis", "[", "numQs_", "]"}], ":=", 
     RowBox[{"Join", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"pauli", "[", 
             RowBox[{"type", ",", " ", "site"}], "]"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"site", ",", "0", ",", 
             RowBox[{"numQs", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"type", ",", 
             RowBox[{"{", 
              RowBox[{"X", ",", "Y", ",", "Z"}], "}"}]}], "}"}]}], "]"}], ",",
          "1"}], "]"}], "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
       RowBox[{"Flatten", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"pauli", "[", 
              RowBox[{"type1", ",", " ", "site1"}], "]"}], ",", 
             RowBox[{"pauli", "[", 
              RowBox[{"type2", ",", " ", "site2"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"site1", ",", "0", ",", 
             RowBox[{"numQs", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"site2", ",", 
             RowBox[{"site1", "+", "1"}], ",", 
             RowBox[{"numQs", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"type1", ",", 
             RowBox[{"{", 
              RowBox[{"X", ",", "Y", ",", "Z"}], "}"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"type2", ",", 
             RowBox[{"{", 
              RowBox[{"X", ",", "Y", ",", "Z"}], "}"}]}], "}"}]}], "]"}], 
         "\[IndentingNewLine]", ",", "3"}], "]"}], "\[IndentingNewLine]", ",",
        "\[IndentingNewLine]", 
       RowBox[{"Flatten", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"pauli", "[", 
              RowBox[{"type1", ",", " ", "site1"}], "]"}], ",", 
             RowBox[{"pauli", "[", 
              RowBox[{"type2", ",", " ", "site2"}], "]"}], ",", 
             RowBox[{"pauli", "[", 
              RowBox[{"type3", ",", " ", "site3"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"site1", ",", "0", ",", 
             RowBox[{"numQs", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"site2", ",", 
             RowBox[{"site1", "+", "1"}], ",", 
             RowBox[{"numQs", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"site3", ",", 
             RowBox[{"site2", "+", "1"}], ",", 
             RowBox[{"numQs", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"type1", ",", 
             RowBox[{"{", 
              RowBox[{"X", ",", "Y", ",", "Z"}], "}"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"type2", ",", 
             RowBox[{"{", 
              RowBox[{"X", ",", "Y", ",", "Z"}], "}"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"type3", ",", 
             RowBox[{"{", 
              RowBox[{"X", ",", "Y", ",", "Z"}], "}"}]}], "}"}]}], "]"}], 
         "\[IndentingNewLine]", ",", "5"}], "]"}]}], "\[IndentingNewLine]", 
      "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"localpaulis", "=", 
     RowBox[{"getListOf3LocalPaulis", "[", "numQs", "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"nCONS", "=", 
     RowBox[{"Length", "@", "localpaulis"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"Print", "@", 
    RowBox[{"Length", "@", "localpaulis"}]}]}]}]], "Input",
 CellChangeTimes->{{3.8606693393006535`*^9, 3.8606693888360643`*^9}, {
  3.860669448593215*^9, 3.8606694558798437`*^9}},
 CellLabel->
  "In[303]:=",ExpressionUUID->"0aeecca7-05d9-4f66-b3bb-b5a9ab5c0127"],

Cell[BoxData["375"], "Print",
 CellChangeTimes->{
  3.860669389074988*^9, 3.8606694569478455`*^9, 3.8606698371771874`*^9, 
   3.86067006330128*^9, {3.8606701275230274`*^9, 3.8606701459127197`*^9}, 
   3.8606701990239353`*^9, 3.860670248731841*^9, 3.8606703090959816`*^9, 
   3.860670340613083*^9, 3.860670439170556*^9, 3.8606704700589533`*^9, 
   3.8606705081789618`*^9, 3.86067062148533*^9, 3.8606706624593253`*^9, 
   3.860670779176828*^9, 3.860671111748681*^9, 3.8606711679453087`*^9, 
   3.860671852769746*^9, 3.860671921028386*^9, {3.8606719732423916`*^9, 
   3.8606719836384*^9}, 3.860672067860314*^9, 3.9029198171744804`*^9, 
   3.9029198764184465`*^9, 3.902919993249456*^9, 3.9029200561967278`*^9, 
   3.902920118637127*^9, 3.9029201914591208`*^9, {3.9029208314068494`*^9, 
   3.9029208434704895`*^9}, 3.9029209060930333`*^9},
 CellLabel->
  "During evaluation of \
In[303]:=",ExpressionUUID->"10cafe59-f516-469e-834b-caca421ffb26"]
}, Open  ]],

Cell[BoxData["\[IndentingNewLine]"], "Input",
 CellChangeTimes->{3.860668965010165*^9},
 CellLabel->
  "In[308]:=",ExpressionUUID->"64560733-7120-4a58-87ea-6d5f81f6c6ba"],

Cell["\<\
Next we define functions for calculating the covariance functions and their \
derivatives given a certain state, Hamiltonian and operator pool\
\>", "Text",
 CellChangeTimes->{{3.8606689035165014`*^9, 3.8606689624333987`*^9}, {
  3.9029197033771944`*^9, 3.9029197057387557`*^9}, {3.902921059995217*^9, 
  3.9029210631838665`*^9}},ExpressionUUID->"3fbdbd04-932f-49d5-866d-\
e888d95028fb"],

Cell[BoxData["\[IndentingNewLine]"], "Input",
 CellChangeTimes->{3.86066887980802*^9},
 CellLabel->
  "In[309]:=",ExpressionUUID->"5f9368cd-0d57-4b2d-b985-0a4ecccc4b99"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"computeFvector", "[", 
    RowBox[{
    "\[Psi]_", ",", " ", "\[Psi]s_", ",", "h\[Psi]_", ",", "hamil_", ",", 
     "operatorPool_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"term2s", ",", "term3", ",", "vec"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Do", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"ApplyCircuit", "[", 
          RowBox[{"\[Psi]", ",", 
           RowBox[{"operatorPool", "[", 
            RowBox[{"[", "k", "]"}], "]"}], ",", 
           RowBox[{"\[Psi]s", "[", 
            RowBox[{"[", "k", "]"}], "]"}]}], "]"}], ";"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "1", ",", 
          RowBox[{"Length", "@", "operatorPool"}]}], "}"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ApplyPauliString", "[", 
       RowBox[{"\[Psi]", ",", "hamil", ",", "h\[Psi]"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"term2s", "=", 
       RowBox[{"CalcInnerProducts", "[", 
        RowBox[{"\[Psi]", ",", "\[Psi]s"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"term3", "=", 
       RowBox[{"CalcInnerProduct", "[", 
        RowBox[{"\[Psi]", ",", "h\[Psi]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"vec", "=", 
       RowBox[{
        RowBox[{"Conjugate", "@", 
         RowBox[{"CalcInnerProducts", "[", 
          RowBox[{"h\[Psi]", ",", "\[Psi]s"}], "]"}]}], "-", 
        RowBox[{"term2s", "*", "term3"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"Re", "@", "vec"}], ",", 
         RowBox[{"Im", "@", "vec"}]}], "]"}], "]"}], ";"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.8606694025280743`*^9, 3.8606694057458577`*^9}, 
   3.860670303570346*^9, {3.9029200166174264`*^9, 3.9029200177473364`*^9}},
 CellLabel->
  "In[310]:=",ExpressionUUID->"c9e021f8-ad2d-4f36-b1c0-47840a142791"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"computeJacobian", "[", 
      RowBox[{
      "\[Psi]_", ",", " ", "\[Psi]s_", ",", "h\[Psi]_", ",", "d\[Psi]_", ",", 
       "hamil_", ",", "operatorPool_", ",", "ansatz_", ",", "params_"}], 
      "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "term1", ",", "term2", ",", "term3", ",", "term3a", ",", "term3b", 
         ",", "en"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"ApplyCircuitDerivs", "[", " ", 
         RowBox[{
          RowBox[{"InitZeroState", "[", "\[Psi]", "]"}], ",", "ansatz", ",", 
          " ", "params", ",", " ", "d\[Psi]"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ApplyCircuit", "[", 
         RowBox[{
          RowBox[{"InitZeroState", "@", "\[Psi]"}], ",", 
          RowBox[{"ansatz", "/.", "params"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ApplyPauliString", "[", 
         RowBox[{"\[Psi]", ",", "hamil", ",", "h\[Psi]"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"ApplyCircuit", "[", 
            RowBox[{"h\[Psi]", ",", 
             RowBox[{"operatorPool", "[", 
              RowBox[{"[", "k", "]"}], "]"}], ",", 
             RowBox[{"\[Psi]s", "[", 
              RowBox[{"[", "k", "]"}], "]"}]}], "]"}], ";"}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", "1", ",", 
            RowBox[{"Length", "@", "operatorPool"}]}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"term1a", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"CalcInnerProducts", "[", 
            RowBox[{
             RowBox[{"d\[Psi]", "[", 
              RowBox[{"[", "k", "]"}], "]"}], ",", "\[Psi]s"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", "1", ",", 
             RowBox[{"Length", "@", "d\[Psi]"}]}], "}"}]}], "]"}]}], "    ", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{"<", "d\[Psi]_k"}], " ", "|", " ", 
          RowBox[{
           RowBox[{
            SubscriptBox["O", "k"], "H", " ", "\[Psi]"}], ">"}]}], " ", 
         "*)"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"ApplyCircuit", "[", 
            RowBox[{"\[Psi]", ",", 
             RowBox[{"operatorPool", "[", 
              RowBox[{"[", "k", "]"}], "]"}], ",", "h\[Psi]"}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"ApplyPauliString", "[", 
            RowBox[{"h\[Psi]", ",", "hamil", ",", 
             RowBox[{"\[Psi]s", "[", 
              RowBox[{"[", "k", "]"}], "]"}]}], "]"}], ";"}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", "1", ",", 
            RowBox[{"Length", "@", "operatorPool"}]}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{"<", "\[Psi]"}], " ", "|", 
          RowBox[{
           SubscriptBox["O", "k"], " ", "H"}], " ", "|", 
          RowBox[{"d\[Psi]_k", ">"}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"term1b", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"Conjugate", "@", 
            RowBox[{"CalcInnerProducts", "[", 
             RowBox[{
              RowBox[{"d\[Psi]", "[", 
               RowBox[{"[", "k", "]"}], "]"}], ",", "\[Psi]s"}], "]"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", "1", ",", 
             RowBox[{"Length", "@", "d\[Psi]"}]}], "}"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"term1", "=", 
         RowBox[{"(", 
          RowBox[{"term1a", "+", "term1b"}], ")"}]}], ";", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"ApplyCircuit", "[", 
            RowBox[{"\[Psi]", ",", 
             RowBox[{"operatorPool", "[", 
              RowBox[{"[", "k", "]"}], "]"}], ",", 
             RowBox[{"\[Psi]s", "[", 
              RowBox[{"[", "k", "]"}], "]"}]}], "]"}], ";"}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", "1", ",", 
            RowBox[{"Length", "@", "operatorPool"}]}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ApplyPauliString", "[", 
         RowBox[{"\[Psi]", ",", "hamil", ",", "h\[Psi]"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"en", "=", 
         RowBox[{"CalcInnerProduct", "[", 
          RowBox[{"\[Psi]", ",", "h\[Psi]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"2", "Re"}], " ", "<", "\[Psi]"}], " ", "|", " ", 
          RowBox[{
           RowBox[{"H", " ", "\[Psi]"}], ">"}]}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"term2", "=", " ", 
         RowBox[{
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"2", 
             RowBox[{"Re", "@", 
              RowBox[{"CalcInnerProducts", "[", 
               RowBox[{
                RowBox[{"d\[Psi]", "[", 
                 RowBox[{"[", "k", "]"}], "]"}], ",", "\[Psi]s"}], "]"}]}]}], 
            ",", 
            RowBox[{"{", 
             RowBox[{"k", ",", "1", ",", 
              RowBox[{"Length", "@", "d\[Psi]"}]}], "}"}]}], "]"}], "*", 
          "en"}]}], "    ", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"2", "Re"}], " ", "<", "d\[Psi]_k"}], " ", "|", " ", 
          RowBox[{
           RowBox[{
            SubscriptBox["H", "a"], " ", "\[Psi]"}], ">"}]}], " ", "*)"}], 
        ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"term3a", "=", 
         RowBox[{"CalcInnerProducts", "[", 
          RowBox[{"\[Psi]", ",", "\[Psi]s"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"2", "Re"}], " ", "<", "\[Psi]"}], " ", "|", " ", 
          RowBox[{
           RowBox[{
            SubscriptBox["H", "a"], " ", "\[Psi]"}], ">"}]}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"term3b", "=", " ", 
         RowBox[{"2", 
          RowBox[{"Re", "[", 
           RowBox[{"CalcInnerProducts", "[", 
            RowBox[{"h\[Psi]", ",", "d\[Psi]"}], "]"}], "]"}]}]}], " ", ";", 
        "\[IndentingNewLine]", 
        RowBox[{"energyGradient", "=", "term3b"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"2", "Re"}], " ", "<", "d\[Psi]_k"}], " ", "|", " ", 
          RowBox[{
           RowBox[{"H", " ", "\[Psi]"}], ">"}]}], " ", "*)"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"term3", "=", 
         RowBox[{"KroneckerProduct", "[", 
          RowBox[{"term3b", ",", "term3a"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"jacobian", "=", 
         RowBox[{"Transpose", "[", 
          RowBox[{"term1", "-", "term2", "-", "term3"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Return", "[", 
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"Re", "@", "jacobian"}], ",", 
           RowBox[{"Im", "@", "jacobian"}]}], "]"}], "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], 
   "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.8606688929421144`*^9, 3.860668893085112*^9}, 
   3.860669408564443*^9, {3.8606702965594664`*^9, 3.8606703000343814`*^9}, {
   3.8606713258008113`*^9, 3.8606713306981354`*^9}, {3.9029197226446533`*^9, 
   3.9029197228917284`*^9}, {3.9029199551337395`*^9, 
   3.9029199702990494`*^9}, {3.9029200261485624`*^9, 3.902920043611808*^9}, {
   3.902920745964837*^9, 3.9029208319743605`*^9}},
 CellLabel->
  "In[311]:=",ExpressionUUID->"0a2bebe8-7650-4fb0-8295-6e04d3b852e3"],

Cell["\<\
Finally we will run CoVaR starting from a random parameter initialisation\
\>", "Text",
 CellChangeTimes->{{3.860671341877982*^9, 
  3.8606713557077727`*^9}},ExpressionUUID->"0b55ffe8-55b9-4437-8f79-\
368a2134c10c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"set", " ", "up", " ", "quantum", " ", "registers"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Psi]", "=", 
     RowBox[{"CreateQureg", "[", "numQs", "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"h\[Psi]", "=", 
     RowBox[{"CreateQureg", "[", "numQs", "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"d\[Psi]", "=", 
     RowBox[{"CreateQuregs", "[", 
      RowBox[{"numQs", ",", "numParams"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[Psi]s", "=", 
     RowBox[{"CreateQuregs", "[", 
      RowBox[{"numQs", ",", "nCONS"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Randomly", " ", "initialise", " ", "parameters", " ", "which", " ", "is",
      " ", "not", " ", "typically", " ", "a", " ", "good", " ", "idea", " ", 
     "but", " ", "will", " ", "be", " ", "fine", " ", "for", " ", "5", " ", 
     "qubits"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"params", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Theta]", "k"], "->", 
        RowBox[{"RandomReal", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "\[Pi]"}], ",", "\[Pi]"}], "}"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "1", ",", "numParams"}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"energyEvo", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"myVecNorms", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"jacobian", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"vec", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.860669486213828*^9, 3.86066956016645*^9}, {
  3.8606696001379023`*^9, 3.8606696337485266`*^9}, {3.8606701186003184`*^9, 
  3.8606701237033978`*^9}, {3.860670457069557*^9, 3.8606704651370316`*^9}, {
  3.8606707070966735`*^9, 3.860670737771515*^9}},
 CellLabel->
  "In[312]:=",ExpressionUUID->"17f2588a-384a-4c33-bd32-48d851762b50"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Monitor", "[", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"Do", "[", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "Create", " ", "the", " ", "ansatz", " ", "state", " ", "\[Psi]"}], 
         " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"ApplyCircuit", "[", 
           RowBox[{
            RowBox[{"InitZeroState", "@", "\[Psi]"}], ",", 
            RowBox[{"ansatz", "/.", "params"}]}], "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"ApplyPauliString", "[", 
           RowBox[{"\[Psi]", ",", "hamil", ",", "h\[Psi]"}], "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"energy", "=", 
           RowBox[{"Chop", "@", 
            RowBox[{"CalcInnerProduct", "[", 
             RowBox[{"h\[Psi]", ",", "\[Psi]"}], "]"}]}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"step", "!=", "1"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"AppendTo", "[", 
              RowBox[{"energyEvo", ",", "energy"}], "]"}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Usually", ",", " ", 
            RowBox[{
            "you", " ", "would", " ", "sample", " ", "from", " ", "the", " ", 
             "operator", " ", "pool"}], ",", " ", 
            RowBox[{
            "but", " ", "for", " ", "5", " ", "qubits", " ", "it", " ", "is", 
             " ", "small", " ", "enough", " ", "that", " ", "we", " ", "can", 
             " ", "use", " ", "all", " ", "of", " ", "it"}]}], " ", "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{"operatorPool", "=", 
           RowBox[{"RandomSample", "[", 
            RowBox[{"localpaulis", ",", " ", "nCONS"}], "]"}]}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"vec", "=", 
           RowBox[{"computeFvector", "[", 
            RowBox[{
            "\[Psi]", ",", "\[Psi]s", ",", "h\[Psi]", ",", "hamil", ",", " ", 
             "operatorPool"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"jacobian", "=", 
           RowBox[{"computeJacobian", "[", 
            RowBox[{
            "\[Psi]", ",", " ", "\[Psi]s", ",", "h\[Psi]", ",", "d\[Psi]", 
             ",", "hamil", ",", "operatorPool", ",", "ansatz", ",", 
             "params"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"myVecNorms", ",", 
            RowBox[{
             RowBox[{"Norm", "[", "vec", "]"}], "^", "2"}]}], "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{
           "Calculate", " ", "step", " ", "from", " ", "jacobian", " ", "and",
             " ", "F", " ", "vector", " ", "with", " ", "regularisation"}], 
           " ", "*)"}], "\[IndentingNewLine]", 
          RowBox[{"Do", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"eta", "=", 
              RowBox[{"0.0001", "*", 
               RowBox[{"2", "^", "etaIT"}]}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"jtj", "=", 
              RowBox[{
               RowBox[{"Transpose", "[", "jacobian", "]"}], ".", 
               "jacobian"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"stepvec", "=", 
              RowBox[{
               RowBox[{"-", "0.5"}], 
               RowBox[{
                RowBox[{"Inverse", "[", 
                 RowBox[{"jtj", "+", 
                  RowBox[{"eta", "*", 
                   RowBox[{"DiagonalMatrix", "@", 
                    RowBox[{"Diagonal", "@", "jtj"}]}]}]}], "]"}], ".", 
                RowBox[{"Transpose", "[", "jacobian", "]"}], ".", "vec"}]}]}],
              ";", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Max", "@", 
                 RowBox[{"Abs", "@", "stepvec"}]}], ">", "1"}], ",", 
               RowBox[{"stepvec", "=", 
                RowBox[{"stepvec", "/", 
                 RowBox[{"Max", "@", 
                  RowBox[{"Abs", "@", "stepvec"}]}]}]}]}], "]"}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"paramsTST", "=", "params"}], ";", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"paramsTST", "[", 
               RowBox[{"[", 
                RowBox[{";;", ",", "2"}], "]"}], "]"}], "=", 
              RowBox[{
               RowBox[{"paramsTST", "[", 
                RowBox[{"[", 
                 RowBox[{";;", ",", "2"}], "]"}], "]"}], "+", "stepvec"}]}], 
             ";", "\[IndentingNewLine]", 
             RowBox[{"ApplyCircuit", "[", 
              RowBox[{
               RowBox[{"InitZeroState", "@", "\[Psi]"}], ",", 
               RowBox[{"ansatz", "/.", "paramsTST"}]}], "]"}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"newNORM", "=", 
              RowBox[{
               RowBox[{"Norm", "[", 
                RowBox[{"computeFvector", "[", 
                 RowBox[{
                 "\[Psi]", ",", "\[Psi]s", ",", "h\[Psi]", ",", "hamil", ",", 
                  " ", "operatorPool"}], "]"}], "]"}], "^", "2"}]}], ";", 
             "\[IndentingNewLine]", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"newNORM", "<", 
                RowBox[{"myVecNorms", "[", 
                 RowBox[{"[", 
                  RowBox[{"-", "1"}], "]"}], "]"}]}], ",", 
               RowBox[{
                RowBox[{"Break", "[", "]"}], ";"}]}], "]"}]}], 
            "\[IndentingNewLine]", ",", 
            RowBox[{"{", 
             RowBox[{"etaIT", ",", "0", ",", "10"}], "}"}]}], "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Update", " ", "parameters"}], " ", "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"params", "[", 
            RowBox[{"[", 
             RowBox[{";;", ",", "2"}], "]"}], "]"}], "=", 
           RowBox[{
            RowBox[{"params", "[", 
             RowBox[{"[", 
              RowBox[{";;", ",", "2"}], "]"}], "]"}], "+", "stepvec"}]}], 
          ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"step", ",", "1", ",", "50"}], "}"}]}], "]"}], ";"}], 
      "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{"Plot", " ", "current", " ", "energy", " ", "and", " ", "Log", 
         RowBox[{"(", "F", ")"}]}], ",", " ", 
        RowBox[{
        "which", " ", "is", " ", "a", " ", "measure", " ", "of", " ", "how", 
         " ", "close", " ", "we", " ", "are", " ", "to", " ", "an", " ", 
         "eigenstate"}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"Row", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"ListLinePlot", "[", 
          RowBox[{"energyEvo", ",", " ", 
           RowBox[{"ImageSize", "\[Rule]", "300"}], ",", " ", 
           RowBox[{"AxesLabel", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"\"\<Iteration\>\"", ",", "\"\<Energy\>\""}], "}"}]}]}], 
          "]"}], ",", 
         RowBox[{"ListLinePlot", "[", 
          RowBox[{
           RowBox[{"Log10", "@", "myVecNorms"}], ",", " ", 
           RowBox[{"ImageSize", "\[Rule]", "300"}], ",", " ", 
           RowBox[{"AxesLabel", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"\"\<Iteration\>\"", ",", "\"\<Log[F]\>\""}], "}"}]}]}], 
          "]"}]}], "}"}], "]"}]}], "\[IndentingNewLine]", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Row", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"ListLinePlot", "[", 
       RowBox[{"energyEvo", ",", " ", 
        RowBox[{"ImageSize", "\[Rule]", "300"}], ",", " ", 
        RowBox[{"AxesLabel", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"\"\<Iteration\>\"", ",", "\"\<Energy\>\""}], "}"}]}]}], 
       "]"}], ",", 
      RowBox[{"ListLinePlot", "[", 
       RowBox[{
        RowBox[{"Log10", "@", "myVecNorms"}], ",", " ", 
        RowBox[{"ImageSize", "\[Rule]", "300"}], ",", " ", 
        RowBox[{"AxesLabel", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"\"\<Iteration\>\"", ",", "\"\<Log10[F]\>\""}], "}"}]}]}], 
       "]"}]}], "}"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.8606696807846327`*^9, 3.8606697823183365`*^9}, {
  3.860669817326255*^9, 3.8606698233863134`*^9}, {3.860669901117053*^9, 
  3.8606699088018456`*^9}, {3.860670029724558*^9, 3.8606700298585587`*^9}, {
  3.860670266364836*^9, 3.860670267919838*^9}, {3.8606703235784693`*^9, 
  3.860670433323556*^9}, {3.8606707507686496`*^9, 3.8606707706098213`*^9}, {
  3.8606708193888206`*^9, 3.8606709162162304`*^9}, {3.860671020394174*^9, 
  3.8606710945392017`*^9}, {3.860671237306985*^9, 3.8606713040607862`*^9}, {
  3.8606719521013823`*^9, 3.860671952247382*^9}, {3.902919782196908*^9, 
  3.9029197829806037`*^9}, {3.902919913685658*^9, 3.9029199417859597`*^9}, {
  3.902921082754114*^9, 
  3.902921142661193*^9}},ExpressionUUID->"d52da49f-7ebc-4e0f-baa9-\
c67b1c75bf94"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{}, {{{}, {}, {
        Hue[0.67, 0.6, 0.6], 
        Directive[
         PointSize[
          NCache[
           Rational[7, 360], 0.019444444444444445`]], 
         RGBColor[0.368417, 0.506779, 0.709798], 
         AbsoluteThickness[1.6]], 
        LineBox[CompressedData["
1:eJxTTMoPSmViYGAwBGIQDQEf7Jf4H+hcN/e6PVTA4buffaS60g0on8PhuKfK
cyvXm1C+gMP5g8fLL5pcgfJFHIpW9byazXEVypdwOCnwdVXsahhfxuF47dq3
HRV3oXwFhx08CVnz7J5B+UoOLAdM5niyPofyVRye3qo8sfPiCyhfzUFyZv2B
RXNgfA2HjpNLq/dKw9RrOVxPrHI6VgYzT8fB7EYyp3IpTL2ew6aaQwdWib+E
8g0cCj4nzs93gvENHRYIMhwu/wXjGzlY3BPQYvN9BeUbO8Rl1YdHWsP4Jg6q
PjJRh4phfFMH3piukG/mML6Zg4qt4cmLs2B8cweTR0k+2adgfAsHjzsSMt92
wPiWDl/cFptvOgDjWznc/NX4KeoxjG/tkO7prDOF8zWUb+NwXllR/aUhjG/r
kCLdaH8+C8a3c9jtxzBdbgGMb+/gZ933W+M0jO/g8CUkM9vnHZTf4OAwd5di
/kfWN1B5R4ei3dcsTglA+Q2ODtcvHReZIQKTd3JYlC95f5soTN7JwX3Zm18s
YjB5ZweeKr0XsjB+g7PDgso2Jk24vIvDg9aXQfpweRcHFpPCCmO4vKtDkeXj
CjO4vKtDeuTcExZweTeH3hPeC6zg8m4O4Wec5G3g8u4Ou7eWRtrC5d0d/omK
xtrB5T0cDEyemNjD5T0cjrr7vgTxAYNe28Y=
         "]]}}}, {{}, {}}}, {
    DisplayFunction -> Identity, DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {
       FormBox["\"Iteration\"", TraditionalForm], 
       FormBox["\"Energy\"", TraditionalForm]}, AxesOrigin -> {0., 0}, 
     DisplayFunction :> Identity, Frame -> {{False, False}, {False, False}}, 
     FrameLabel -> {{None, None}, {None, None}}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], ImageSize -> 300, 
     Method -> {
      "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
       "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           Identity[
            Part[#, 1]], 
           Identity[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           Identity[
            Part[#, 1]], 
           Identity[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{0., 49}, {0, 0.8777160221745751}}, PlotRangeClipping -> 
     True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.05]}}, Ticks -> {Automatic, Automatic}}], 
   GraphicsBox[{{}, {{{}, {}, {
        Hue[0.67, 0.6, 0.6], 
        Directive[
         PointSize[
          NCache[
           Rational[1, 60], 0.016666666666666666`]], 
         RGBColor[0.368417, 0.506779, 0.709798], 
         AbsoluteThickness[1.6]], 
        LineBox[CompressedData["
1:eJw1yWtIkwEUxvFpt0mmc1vgslouKxsWZmpquT266ZwEWZGGUXZBDQ0FGySo
9UKhH4pYIlG4zGVjNcgchZAf3MAoBZk2LSMxNlZibg6tLa+5ip33wOHwO//Y
i9UnSkM5HE7Sv/1/gzMrPzta4ky5uiinBzhPk5W3Hi6QuQiobvcOXWDNQ7G1
ybAqYi1EWd2UvdY9T47GJmGPPew56604ebSuWc+w3oHlGlfm0rSfLMGT3xNr
bGYfOQ6RodVOSd9P8m7ENe6VpW+eIcej0KS1pHx1kaU4XX8+V3GddQJaKrtH
I9qd5P24VlslN/YNkhMRs/K6q9FjJh9A2baZXUaNjpyEiqgr8571PZagD2Lk
jtQkKnhLTsYpLfO4SD1ATsGzwJBftmGYnIpxwfRgVsEH8iHcM4rba9xfyGlY
1Cbo51bHyelIzL9b9Oqdi5yBR0rvvspPk+TD6OR2xKerPOQjOF4+MNbW4CVn
ot/WGr3TO0uWocVXHj7AzJHlSLsZFqVo+EUGttd/G35p8QXNAMJw88eSXj/1
LFgMjpzOiQXqWehd3JO2kv+HejaYjrZ1ZU0h1mDPhtixIPF71wbNUUA3ohF8
LuRSV6C7uH/IUr2RuhI2/X27OCaCuhJbHK2aGyIe9RyUBpabM8bITA70OuHy
sTNR1HMhecB3xlrITC5mQ15ULPH51FXwinimrnNkRoU305fe6w1sz8N39aRe
9oPteZCL3JGpUgF1NarcLVPuywLrX1I/98M=
         "]]}}}, {{}, {}}}, {
    DisplayFunction -> Identity, DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {
       FormBox["\"Iteration\"", TraditionalForm], 
       FormBox["\"Log10[F]\"", TraditionalForm]}, AxesOrigin -> {0., 0}, 
     DisplayFunction :> Identity, Frame -> {{False, False}, {False, False}}, 
     FrameLabel -> {{None, None}, {None, None}}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], ImageSize -> 300, 
     Method -> {
      "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
       "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           Identity[
            Part[#, 1]], 
           Identity[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           Identity[
            Part[#, 1]], 
           Identity[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{0., 50}, {-4.852462419595083, 1.59062755619859}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, Ticks -> {Automatic, Automatic}}]},
  "RowDefault"]], "Output",
 CellChangeTimes->{
  3.8606710273122053`*^9, 3.8606711057162156`*^9, 3.860671160049677*^9, 
   3.8606712166963162`*^9, 3.860671901933742*^9, {3.860671968383381*^9, 
   3.860671980140383*^9}, 3.860672029780383*^9, 3.860672115682306*^9, {
   3.902919872547753*^9, 3.9029198868563824`*^9}, 3.902919997450371*^9, 
   3.9029201086015263`*^9, 3.9029201775912623`*^9, 3.902920241383613*^9, 
   3.902920837262787*^9, 3.902920895004329*^9, 3.9029209563656244`*^9},
 CellLabel->
  "Out[322]=",ExpressionUUID->"0f46f94f-7d76-4308-bb62-ecdd0f143735"]
}, Open  ]],

Cell["", "Text",
 CellChangeTimes->{{3.860670920155758*^9, 
  3.860670920487755*^9}},ExpressionUUID->"e8563f86-e1b7-4aee-ad5a-\
d114c4123ab8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "See", " ", "how", " ", "close", " ", "in", " ", "energy", " ", "the", " ",
     "root", " ", "finding", " ", "got", " ", "to", " ", "an", " ", 
    "eigenenergy"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"finalEnergy", "=", 
     RowBox[{"energyEvo", "[", 
      RowBox[{"[", 
       RowBox[{"-", "1"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"eDiff", "=", 
    RowBox[{"finalEnergy", " ", "-", " ", 
     RowBox[{"Nearest", "[", 
      RowBox[{"eigs", ",", " ", "finalEnergy"}], "]"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.860670926573216*^9, 3.8606709971537*^9}},
 CellLabel->
  "In[323]:=",ExpressionUUID->"d5b5b960-6c07-47e2-b858-a470d6d7cbd7"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"-", "2.1773085303600226`*^-6"}], "}"}]], "Output",
 CellChangeTimes->{
  3.8606709978777018`*^9, 3.8606711601526775`*^9, 3.8606712167933064`*^9, 
   3.860671902029778*^9, 3.860671968493389*^9, 3.860672029884384*^9, 
   3.860672115788309*^9, 3.902919873166896*^9, 3.902920108653512*^9, 
   3.9029201776382627`*^9, {3.9029202414326153`*^9, 3.902920267692255*^9}, 
   3.9029208950622954`*^9, 3.9029209564159217`*^9},
 CellLabel->
  "Out[324]=",ExpressionUUID->"df994958-fe97-47a1-b375-492806c563d5"]
}, Open  ]]
}, Open  ]]
},
WindowSize->{574.8, 573.6},
WindowMargins->{{-4.8, Automatic}, {Automatic, 0}},
FrontEndVersion->"13.0 for Microsoft Windows (64-bit) (February 4, 2022)",
StyleDefinitions->"Default.nb",
ExpressionUUID->"f44c1789-6638-44ae-8079-133c292a3d95"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 157, 3, 98, "Title",ExpressionUUID->"1a68cbf2-ea48-4487-aff9-fc9ec31337b5"],
Cell[740, 27, 341, 7, 85, "Subtitle",ExpressionUUID->"00f3e1a7-c8ac-4c69-bccc-6d9ce8570bfd"],
Cell[1084, 36, 729, 18, 105, "Input",ExpressionUUID->"957f1326-ac59-4622-a6ef-d5d9fdff11e5"],
Cell[1816, 56, 2239, 56, 128, "Text",ExpressionUUID->"8eeba43a-a171-43fd-b1ca-a19cef20426b"],
Cell[CellGroupData[{
Cell[4080, 116, 8451, 215, 772, "Input",ExpressionUUID->"3af22d37-ea72-486f-8724-2e969b02ce9d"],
Cell[CellGroupData[{
Cell[12556, 335, 20634, 457, 192, "Print",ExpressionUUID->"454199c5-45af-41eb-8639-299cd6ac9b4e"],
Cell[33193, 794, 995, 15, 22, "Print",ExpressionUUID->"19b8dbac-7d12-4a69-af2e-58eaec6eaa1c"]
}, Open  ]]
}, Open  ]],
Cell[34215, 813, 2738, 69, 197, "Text",ExpressionUUID->"caff12e3-ad50-48cc-8b36-c8b8643e36ed"],
Cell[CellGroupData[{
Cell[36978, 886, 4183, 104, 448, "Input",ExpressionUUID->"0aeecca7-05d9-4f66-b3bb-b5a9ab5c0127"],
Cell[41164, 992, 940, 15, 22, "Print",ExpressionUUID->"10cafe59-f516-469e-834b-caca421ffb26"]
}, Open  ]],
Cell[42119, 1010, 170, 3, 48, "Input",ExpressionUUID->"64560733-7120-4a58-87ea-6d5f81f6c6ba"],
Cell[42292, 1015, 397, 7, 58, "Text",ExpressionUUID->"3fbdbd04-932f-49d5-866d-e888d95028fb"],
Cell[42692, 1024, 169, 3, 48, "Input",ExpressionUUID->"5f9368cd-0d57-4b2d-b985-0a4ecccc4b99"],
Cell[42864, 1029, 2035, 52, 200, "Input",ExpressionUUID->"c9e021f8-ad2d-4f36-b1c0-47840a142791"],
Cell[44902, 1083, 8017, 197, 719, "Input",ExpressionUUID->"0a2bebe8-7650-4fb0-8295-6e04d3b852e3"],
Cell[52922, 1282, 225, 5, 35, "Text",ExpressionUUID->"0b55ffe8-55b9-4437-8f79-368a2134c10c"],
Cell[53150, 1289, 2291, 60, 295, "Input",ExpressionUUID->"17f2588a-384a-4c33-bd32-48d851762b50"],
Cell[CellGroupData[{
Cell[55466, 1353, 9428, 207, 1132, "Input",ExpressionUUID->"d52da49f-7ebc-4e0f-baa9-c67b1c75bf94"],
Cell[64897, 1562, 5167, 110, 395, "Output",ExpressionUUID->"0f46f94f-7d76-4308-bb62-ecdd0f143735"]
}, Open  ]],
Cell[70079, 1675, 142, 3, 35, "Text",ExpressionUUID->"e8563f86-e1b7-4aee-ad5a-d114c4123ab8"],
Cell[CellGroupData[{
Cell[70246, 1682, 788, 19, 105, "Input",ExpressionUUID->"d5b5b960-6c07-47e2-b858-a470d6d7cbd7"],
Cell[71037, 1703, 536, 10, 35, "Output",ExpressionUUID->"df994958-fe97-47a1-b375-492806c563d5"]
}, Open  ]]
}, Open  ]]
}
]
*)

